import { Injectable } from '@nestjs/common';
import { Repository } from 'typeorm';
import { Swipe } from './entities/swipe.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { UsersService } from 'src/users/users.service';
import { UserIdNotExistsException } from 'src/utils/errors';
import { PairsService } from 'src/pairs/pairs.service';

@Injectable()
export class SwipesService {
  constructor(
    @InjectRepository(Swipe)
    private readonly swipesRepository: Repository<Swipe>,
    private readonly usersService: UsersService,
    private readonly pairsService: PairsService,
  ) {}

  async createSwipe(
    sourceUserId: number,
    targetUserId: number,
    liked: boolean,
  ) {
    const users = await this.usersService.getByIds([
      sourceUserId,
      targetUserId,
    ]);
    if (users.length !== 2) {
      throw new UserIdNotExistsException();
    }

    const swipe = this.swipesRepository.create({
      sourceUser: { id: sourceUserId },
      targetUser: { id: targetUserId },
      liked,
    });
    await this.swipesRepository.save(swipe);

    if (liked) {
      const answeringLikeSwipe = await this.swipesRepository.findBy({
        sourceUser: { id: targetUserId },
        targetUser: { id: sourceUserId },
      });
      if (answeringLikeSwipe) {
        await this.pairsService.createPair([sourceUserId, targetUserId]); // TODO use transaction?
      }
    }

    return swipe;
  }

  async getSwipesByUser(sourceUserId: number) {
    const user = await this.usersService.getById(sourceUserId);
    if (!user) {
      throw new UserIdNotExistsException();
    }
    return this.swipesRepository.findBy({ sourceUser: { id: sourceUserId } });
  }
}
